-- simple library to work with SavedVariables
-- credits: Another
-- example
--[[
	local storage = LibStub("EasyStorage-1.1", false)

	MyAddon_SavedVariablesPerCharacter = MyAddon_SavedVariablesPerCharacter or {}
	MyAddonPerChar = storage:New(MyAddon_SavedVariablesPerCharacter)
	...
	MyAddon_SavedVariables = MyAddon_SavedVariables or {}
	MyAddonGlobal = storage:New(MyAddon_SavedVariables)
	...
	value1 = MyAddonPerChar:Get({"node_root", "node2"}, true)
	value2 = MyAddonGlobal:Get({"node_root", "node3"}, 10)
	...
	MyAddonPerChar:Set({"node_root", "node2"}, some_value)
	MyAddonGlobal:Set({"node_root", "node3"}, 20)
]]

local MAJOR, MINOR = "EasyStorage-1.1", 0
local EasyStorage = LibStub:NewLibrary(MAJOR, MINOR)

if not EasyStorage then return end

EasyStorageClass = EasyStorageClass or {}
local ESClass = EasyStorageClass
ESClass.__index = ESClass

function EasyStorage:New(storage)
	local instance = {}
	setmetatable(instance, ESClass)
	instance.storage = storage
	return instance
end

function EasyStorageClass:Get(keys_arr, default)
	local sv = self.storage
	for i = 1, #keys_arr do
		local v = keys_arr[i]
		if (type(sv) == "table") and (sv[v] ~= nil) then
			sv = sv[v]
		else
			return default
		end
	end
	return sv
end

local function CheckIsEmpty(value)
	if (type(value) == "table") then
		-- check is simple array first
		if #value > 0 then
			return false
		end
		-- hashtable
		for k, v in pairs(value) do
			if v then
				return false
			end
		end
		return true
	else -- not a table
		return false
	end
end

function EasyStorageClass:Set(keys_arr, value)
	local sv = self.storage
	for i = 1, #keys_arr do
		local v = keys_arr[i]
		if i == #keys_arr then
			sv[v] = value
		else
			sv[v] = sv[v] or {}
			sv = sv[v]
		end
	end
	
	if value == nil then
		-- make path to last node
		local temp_keys = {}
		for i = 1, #keys_arr - 1 do
			temp_keys[i] = keys_arr[i]
		end
		-- get node
		local temp = self:Get(temp_keys, nil)
		if temp then
			if CheckIsEmpty(temp) then
				self:Set(temp_keys, nil)
			else
				return
			end
		end
	end
end
